@@ -21,6 +21,7 @@ class CategoricalAutoEncoder(BaseEncoder):
         self.net = None
         self.encoder = None
         self.decoder = None
+        self.predict_proba = None # whether to return the belief distribution as well
         self.onehot_encoder = OneHotEncoder(is_target=self.is_target)
         self.desired_error = 0.01
         self.use_autoencoder = None
@@ -110,11 +111,11 @@ class CategoricalAutoEncoder(BaseEncoder):
                 return embeddings
 
     def decode(self, encoded_data):
+        self.onehot_encoder.predict_proba = self.predict_proba
         if not self.use_autoencoder:
             return self.onehot_encoder.decode(encoded_data)
         else:
             with torch.no_grad():
                 oh_encoded_tensor = self.decoder(encoded_data)
                 oh_encoded_tensor = oh_encoded_tensor.to('cpu')
-                decoded_categories = self.onehot_encoder.decode(oh_encoded_tensor)
-                return decoded_categories
+                return self.onehot_encoder.decode(oh_encoded_tensor)
